package com.blogspot.thedsweb.main;

import java.net.MalformedURLException;

import com.blogspot.thedsweb.engine.Brightness;
import com.blogspot.thedsweb.util.Battery;
import com.blogspot.thedsweb.util.Config;
import com.blogspot.thedsweb.util.Files;
import com.blogspot.thedsweb.util.Initialization;
import com.blogspot.thedsweb.util.SingleInstance;
import com.blogspot.thedsweb.util.Translate;

public class Yawls {
	public static void main(String[] args) {
		//Simple command line argument parsing
		if ( SingleInstance.lockInstance() ) {
			for( String options : args ){
				switch(options) {
				case "-h":
				case "--help":
					help();
					break;
					//
				case "-r":
				case "--restore":
					if ( rootCheck() ) {
						restoreConfig();
					} else {
						System.out.println(Translate._("This command must be run as root"));
					}
					break;
					//
				case "-c":
				case "--cron":
					if ( rootCheck() ) {
						try {
							automatic();
						} catch (MalformedURLException e) {
							e.printStackTrace();
						}
					} else {
						System.out.println(Translate._("This command must be run as root"));
					}
					break;
					//
				case "-d":
				case "--daemon":
					if ( rootCheck() ) {
						try {
							daemon();
						} catch (MalformedURLException e) {
							e.printStackTrace();
						}
					} else {
						System.out.println(Translate._("This command must be run as root"));
					}
					break;
					//
				default:
					System.out.println(Translate._("Usage: yawls [OPTIONS]"));
					//
				}
			}
			if ( args == null || args.length == 0 ){
				System.out.println(Translate._("Usage: yawls [OPTIONS]"));
			}
		} else {
			System.out.println(Translate._("Another instance of yawls is already running!"));
		}
	}
	
	public static void help(){
		System.out.println(
				  Translate._("Usage: yawls") + "\n"
				+ Translate._("yawls adjust the brightness level of your display by using\nthe internal/externel webcam of your notebook as an ambient\nlight sensor.")
				+ "\n\n"
				+ Translate._("If you wish to change the interval of ambient light checks\nyou need to modify the file:") + "\n"
				+ "   /etc/cron.d/yawls"
				+ "\n\n"
				+ Translate._("Options:") + "\n"
				+ "  " + Translate._("--cron\t or -c use webcam as brightness reference") + "\n"
				+ "  " + Translate._("--help\t or -h show this help") + "\n"
				+ "  " + Translate._("--restore\t or -r restore config files"));
	}
	
	public static void automatic() throws MalformedURLException {
		//Read configuration File
		Config config = new Config();
		
		//Get current brightness and save it
		Brightness brightness = new Brightness(config);
		
		// If the current value change in a extreme way
		// try to set it again
		setNewIfChangeTooExtreme(brightness);
		
		//Set new brightness level
		brightness.setBrightness();
	}
	
	public static void daemon() throws MalformedURLException {
		//Set daemon mode
		System.out.println(Translate._("Yawls_service_started_press_{ctrl+c}_to_abort..."));
		Runtime.getRuntime().addShutdownHook( new Thread() {
			@Override 
			public void run() {
				Files.deletePid();
			}
		} );
		System.out.close();
		System.err.close();
		
		//Initialize Files if necessary 
		Initialization.Files();
		
		//Values to increase loop sleep if nothing to do
		int before, after;
		int maxIncrease = 500;
		int minIncrease = 100;
		int increase = maxIncrease;
		
		//Upper and lower limit between brightness capturing
		int upperLimit = 30000;
		int lowerLimit = 500;
		int[] limit = {500, 30000, 500, 30000};
		int i = lowerLimit;
		
		//Read configuration File
		Config config = new Config();
		boolean faceDetectConfig = config.faceDetect();
		lowerLimit = config.lowerLimit();
		upperLimit = config.upperLimit();
		minIncrease = config.minIncrease();
		maxIncrease = config.maxIncrease();
		
		//Get current brightness in brightness object
		Brightness brightness = new Brightness(config);
		
		//Initialize variable for face detection
		boolean faceDetect;
		
		//Loop until interrupt
		while ( true ) {
			//Get current brightness
			brightness.setCurrent();
			
			// If the current value change in a extreme way
			// try to set it again
			setNewIfChangeTooExtreme(brightness);
			
			//Get last brightness value BEFORE processing
			before = brightness.getLast();
			
			/* If face detection is deactivated then set
			 * faceDetect to true only else set
			 * faceDetect to the value of the getFace function
			 * of Brightness object
			*/ 
			if ( faceDetectConfig ) {
				faceDetect = brightness.getFace();
			} else {
				faceDetect = true;
			}
			
			// Check upper and lower limit and set if
			// necessary a more battery friendly value
			limit = checkLimit(limit);
			lowerLimit = limit[2];
			upperLimit = limit[3];
			
			if ( faceDetect ) {
				//Set brightness level
				brightness.setBrightness();
				upperLimit = limit[1];
				increase = maxIncrease;
			} else {
				//Dim display
				brightness.darkenBrightness();
				increase = minIncrease;
			}
			
			//Get last brightness value AFTER processing
			after = brightness.getLast();
			
			//Increase loop sleep
			if ( before == after && i < upperLimit ) {
				i = i + increase;
			} else {
				i = lowerLimit;
			}
			
			//Sleep for 0.5+ seconds
			try {
				Thread.sleep(i);
			} catch ( InterruptedException e ) {}
		}
	}
	
	public static int[] checkLimit(int[] limit) {
		int lowerLimitConst = limit[0];
		int upperLimitConst = limit[1];
		int lowerLimit = limit[2];
		int upperLimit = limit[3];
		
		//Upper limit correction
		if ( lowerLimitConst > upperLimitConst ) {
			upperLimitConst = lowerLimit;
			upperLimit = lowerLimit;
		}
		
		//Battery friendliness 
		if ( Battery.status() > 30 ) {
			lowerLimit = lowerLimitConst;
			upperLimit = upperLimitConst;
		} else {
			lowerLimit = ((upperLimitConst-lowerLimitConst)/2)+lowerLimitConst;
		}
		
		//Return new upper and lower limit
		limit[1] = upperLimitConst;
		limit[2] = lowerLimit;
		limit[3] = upperLimit;
		return limit;
	}
	
	public static void setNewIfChangeTooExtreme(Brightness brightness) {
		// Set new current if the change between the last and
		// current values is very high 
		if ( brightness.control() ) {
			brightness.setCurrent();
		}
	}
	
	public static void restoreConfig(){
		//Forces initialization of saved files
		Initialization.forceFiles();
	}
	
	public static boolean rootCheck() {
		//Check if the current user has root privileges
		return System.getProperty("user.name").equalsIgnoreCase("root");
	}
}
